/*==========================
jquery.soap.js  http://plugins.jquery.com/soap/ or https://github.com/doedje/jquery.soap
version: 1.2.1

jQuery plugin for communicating with a web service using SOAP.

One function to send the soapRequest that takes a complex object as a parameter

Dependencies
------------
jQuery -- built and tested with v1.9.1 and v1.10.1, MAY work back to v1.6
SOAPResponse.toJSON() depends on jQuery.xml2json.js

Authors / History
-----------------

2013-06 >> fix for SOAPServer and SOAPAction headers, better params object to SOAPObject function
Remy Blom == www.hku.nl == remy.blom@kmt.hku.nl
Utrecht School of Arts,The Netherlands

2013-03 >> update internal OO structure, enable XML & object input as well as JSON
Zach Shelton == zachofalltrades.net
https://github.com/zachofalltrades/jquery.soap

2013-02-19 >> published to plugins.jquery.com/soap/
Remy Blom == https://github.com/doedje/jquery.soap

2011-10-31 >> fix handling of arrays in JSON paramaters
Diccon Towns == dtowns@reapit.com

2009-12-03 >> wrap jqSOAPClient as plugin
Remy Blom == www.hku.nl == remy.blom@kmt.hku.nl
Utrecht School of Arts,The Netherlands

2007-12-20 >> jqSOAPClient.beta.js by proton17
http://archive.plugins.jquery.com/project/jqSOAPClient

License
-------

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

I may consider permitting uses outside of the license terms on a by-case basis.

USAGE
-----

options {
	url: 'http://my.server.com/soapservices/',		//endpoint address for the service
	method: 'helloWorld',							// service operation name
													// 1) will be appended to url if appendMethodToURL=true
													// 2) will be used for request element name when building xml from JSON 'params' (unless 'elementName' is provided)
	appendMethodToURL: true,						// method name will be appended to URL defaults to true
	SOAPAction: 'action',							// manually set the Request Header 'SOAPAction', defaults to the method specified above (optional)
	soap12: false,									// use SOAP 1.2 namespace and HTTP headers - default to false

	//params can be XML DOM, XML String, or JSON
	params: domXmlObject,							// XML DOM object
	params: xmlString,								// XML String for request (alternative to internal build of XML from JSON 'params')
	params: {										// JSON structure used to build request XML - SHOULD be coupled with ('namespaceQualifier' AND 'namespaceURL') AND ('method' OR 'elementName')
		name: 'Remy Blom',
		msg: 'Hi!'
	},

	//these options ONLY apply when the request XML is going to be built from JSON 'params'
	namespaceQualifier: 'myns',						// used as namespace prefix for all elements in request (optional)
	namespaceURL: 'urn://service.my.server.com',	// namespace url added to parent request element (optional)
	noPrefix: false,								// set to true if you don't want the namespaceQualifier to be the prefix for the nodes in params. defaults to false (optional)
	elementName: 'requestElementName',				// override 'method' as outer element (optional)

	// WS-Security
	wss: {
		username: 'user',
		password: 'pass',
		nonce: 'w08370jf7340qephufqp3r4',
		created: new Date().getTime()
	},

	//callback functions
	request: function (SOAPRequest)  {},			// callback function - request object is passed back prior to ajax call (optional)
	success: function (SOAPResponse) {},			// callback function to handle successful return (required)
	error:   function (SOAPResponse) {},				// callback function to handle fault return (required)

	// debugging
	enableLogging: false						// to enable the local log function set to true, defaults to false (optional)
}

======================*/
(function(e){function u(n){t&&typeof console=="object"&&e.isFunction(console.log)&&console.log(n)}function a(t){typeof console=="object"&&e.isFunction(console.warn)&&console.warn(t)}var t;e.soap=function(s){var f={};this.globalConfig||(this.globalConfig={appendMethodToURL:!0,noPrefix:!1,soap12:!1,enableLogging:!1});if(s&&!s.params){e.extend(this.globalConfig,s);return}e.extend(f,this.globalConfig,s),t=f.enableLogging,u(f);var l;if(e.type(f.params)==="string"){if(/\S/.test(f.params)){var c;r.isWrappedWithEnvelope(f.params)?c=f.params:c=r.wrapWithEnvelope(f.params,f.soap12),l=new i,l.toString=function(){return c}}}else if(e.isXMLDoc(f.params))l=f.params,l=new i,l.toString=function(){return r.dom2String(f.params)};else if(e.isPlainObject(f.params)&&(!!f.method||!!f.elementName)){var h=f.elementName?f.elementName:f.method,p=!!f.namespaceQualifier&&!f.noPrefix?f.namespaceQualifier+":":"",d=r.json2soap(h,f.params,p);!f.namespaceURL&&!!f.namespaceUrl&&(a("jquery.soap: namespaceUrl is deprecated, use namespaceURL instead!"),f.namespaceURL=f.namespaceUrl),!!f.namespaceQualifier&&!!f.namespaceURL&&(d.ns=r.Namespace(f.namespaceQualifier,f.namespaceURL)),l=new i(d),f.soap12&&(l.soapNamespace=r.SOAP12_NAMESPACE)}if(!!f.wss&&!!f.wss.username&&!!f.wss.password){var v=new o("wsse:Security"),m=new o("wsse:UsernameToken"),g=new o("wsse:Username"),y=new o("wsse:Password"),b=new o("wsse:Nonce"),w=new o("wsu:Created");v.attr("xmlns:wsse","http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"),v.attr("xmlns:wsu","http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"),g.attr("Type","http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"),y.attr("Type","http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText"),b.attr("Type","http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"),w.attr("Type","http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"),g.val(f.wss.username),y.val(f.wss.password),b.val(f.wss.nonce),w.val(f.wss.created),m.appendChild(g),m.appendChild(y),m.appendChild(b),m.appendChild(w),v.appendChild(m),l.addHeader(v)}!!l&&e.isFunction(f.request)&&f.request(l);if(!!f.url&&!!l){if(!e.isFunction(f.error))throw new Error("error callback function was not provided");if(!e.isFunction(f.success))throw new Error("success callback function was not provided");var E=new n;E.Proxy=f.url,f.appendMethodToURL&&!!f.method&&(E.Proxy+=f.method);var S=f.method;!f.SOAPAction||(S=f.SOAPAction),E.SendRequest(S,l,function(e){u(e),e.status!=="success"?f.error(e):f.success(e)})}};var n=function(){this.typeOf="SOAPClient",this._tId=null,this.Proxy="",this.SOAPServer="",this.CharSet="UTF-8",this.Timeout=0,this.SendRequest=function(t,n,i){if(!e.isFunction(i))throw new Error("callback function was not specified");if(!!this.Proxy){var o=n.toString(),a=r.SOAP11_TYPE;r.isSOAP12(o)&&(a=r.SOAP12_TYPE);var f=this.SOAPServer,l=e.ajax({type:"POST",url:this.Proxy,dataType:"xml",processData:!1,data:o,contentType:a+"; charset="+this.CharSet,beforeSend:function(e){e.setRequestHeader("Method","POST"),e.setRequestHeader("SOAPServer",f),a===r.SOAP11_TYPE&&e.setRequestHeader("SOAPAction",t)}});l.always(function(t,n,o){var a,f="data",l="jqXHR";u("status: "+e.type(n)+" '"+n+"'"),n!=="success"&&(f="jqXHR",l="errorThrown"),u("-- a: "+f+" --"),u(t),u("-- c: "+l+" --"),u(o),u("a: "+f+": "+e.type(t)+"  isXMLDoc(a):"+e.isXMLDoc(t)+"  a.responseText:"+e.type(t.responseText)+"  isXMLDoc(a.responseText):"+e.isXMLDoc(t.responseText)+"  a.responseXML:"+e.type(t.responseXML)+"  isXMLDoc(a.responseXML):"+e.isXMLDoc(t.responseXML)),u("c: "+l+": "+e.type(o)+"  isXMLDoc(c):"+e.isXMLDoc(o)+"  c.responseText:"+e.type(o.responseText)+"  isXMLDoc(c.responseText):"+e.isXMLDoc(o.responseText)+"  c.responseXML:"+e.type(o.responseXML)+"  isXMLDoc(c.responseXML):"+e.isXMLDoc(o.responseXML));if(e.isXMLDoc(t))a=new s(n,o),a.content=r.dom2string(t);else if(e.type(t)==="object")a=new s(n,t),a.content=t.responseXML===undefined?t.responseText:t.responseXML;else{if(e.type(o)!=="object")throw new Error("unexpected return types for jqXHR.always()");a=new s(n,o),a.content=o.responseXML===undefined?o.responseText:o.responseXML}i(a)})}}},r=function(){var e={SOAP11_TYPE:"text/xml",SOAP12_TYPE:"application/soap+xml",SOAP11_NAMESPACE:"http://schemas.xmlsoap.org/soap/envelope/",SOAP12_NAMESPACE:"http://www.w3.org/2003/05/soap-envelope",isSOAP11:function(e){return e.indexOf(r.SOAP11_NAMESPACE)!==-1},isSOAP12:function(e){return e.indexOf(r.SOAP12_NAMESPACE)!==-1},isWrappedWithEnvelope:function(e){return this.isSOAP11(e)||this.isSOAP12(e)},wrapWithEnvelope:function(e,t){var n=t?this.SOAP12_NAMESPACE:this.SOAP11_NAMESPACE,r='<soap:Envelope xmlns:soap="'+n+'"><soap:Body>'+e+"</soap:Body></soap:Envelope>";return r},json2soap:function(e,t,n,r){var i,s;if(typeof t=="object")if(t.constructor.toString().indexOf("Array")!=-1)for(var u in t)s=this.json2soap(e,t[u],n,r),r.appendChild(s);else{i=new o(n+e);for(var a in t)s=this.json2soap(a,t[a],n,i),i.appendChild(s)}else i=new o(n+e),i.val(""+t);return i},Namespace:function(e,t){return{name:e,uri:t}},dom2string:function(e){return typeof XMLSerializer!="undefined"?(new window.XMLSerializer).serializeToString(e):e.xml},soap2xml:function(e){var t=[],n=!1;try{if(!!e&&typeof e=="object"&&e.typeOf==="SOAPObject"){t.push("<"+e.name),!e.ns||(typeof e.ns=="object"?(n=!0,t.push(" xmlns:"+e.ns.name+'="'+e.ns.uri+'"')):t.push(' xmlns="'+e.ns+'"'));if(e.attributes.length>0){var i,s=e.attributes.length-1;do i=e.attributes[s],n?t.push(" "+e.ns.name+":"+i.name+'="'+i.value+'"'):t.push(" "+i.name+'="'+i.value+'"');while(s--)}t.push(">");if(e.hasChildren()){var o,u;for(o in e.children)u=e.children[o],typeof u=="object"&&t.push(r.soap2xml(u))}return!e.value||t.push(e.value),t.push("</"+e.name+">"),t.join("")}}catch(a){alert("Unable to process SOAPObject! Object must be an instance of SOAPObject")}}};return e}(),i=function(e){this.typeOf="SOAPRequest",this.soapNamespace=r.SOAP11_NAMESPACE;var t=[],n=[],i=e?[e]:[];this.addNamespace=function(e,n){t.push(new r.Namespace(e,n))},this.addHeader=function(e){n.push(e)},this.addBody=function(e){i.push(e)},this.toString=function(){var e=new o("soap:Envelope");e.attr("xmlns:soap",this.soapNamespace);if(t.length>0){var r,s;for(r in t)if(!t.hasOwnProperty||t.hasOwnProperty(r))s=t[r],typeof s=="object"&&e.attr("xmlns:"+s.name,s.uri)}if(n.length>0){var u=e.appendChild(new o("soap:Header")),a;for(a in n)(!n.hasOwnProperty||n.hasOwnProperty(a))&&u.appendChild(n[a])}if(i.length>0){var f=e.appendChild(new o("soap:Body")),l;for(l in i)(!i.hasOwnProperty||i.hasOwnProperty(l))&&f.appendChild(i[l])}return e.toString()}},s=function(t,n){this.typeOf="SOAPResponse",this.status=t,this.headers=n.getAllResponseHeaders().split("\n"),this.httpCode=n.status,this.httpText=n.statusText,this.content=null,this.toString=function(){if(typeof this.content=="string")return this.content;if(e.isXMLDoc(this.content))return r.dom2string(this.content);throw new Error("Unexpected Content: "+e.type(this.content))},this.toXML=function(){return e.isXMLDoc(this.content)?this.content:e.parseXML(this.content)},this.toJSON=function(){if(e.xml2json)return e.xml2json(this.content);throw new Error("Missing JQuery Plugin 'xml2json'")}},o=function(e){this.typeOf="SOAPObject",this.ns=null,this.name=e,this.attributes=[],this.children=[],this.value=null,this.attr=function(e,t){return this.attributes.push({name:e,value:t}),this},this.appendChild=function(e){return this.children.push(e),e},this.addParameter=function(e,t){var n=new o(e);n.val(t),this.appendChild(n)},this.hasChildren=function(){return this.children.length>0?!0:!1},this.val=function(e){return e?(this.value=e,this):this.value},this.toString=function(){return r.soap2xml(this)}}})(jQuery);